@@ -2,9 +2,10 @@
 """
 Module containing command for generating keys on the card
 """
+import random
 import re
 from pathlib import Path
-from typing import Union
+from typing import Union, Tuple
 
 import cryptnoxpy
 from tabulate import tabulate
@@ -17,6 +18,8 @@ from .helper import (
     security,
     ui
 )
+from .helper.backup import DEFAULT_REGIONS
+from .helper.download_folder import get_download_folder
 
 try:
     import enums
@@ -62,25 +65,38 @@ class Seed(Command):
         return result
 
     @staticmethod
-    def _backup(card: cryptnoxpy.Card) -> int:
-        pin_code = Seed._get_pin_code(card)
+    def _backup(card: cryptnoxpy.Card, pin_code: str = None, seed: bytes = None) -> int:
+        pin_code = pin_code or Seed._get_pin_code(card)
+
+        credentials_file = Path(get_download_folder()).joinpath('rootkey.csv')
+        access_key_id, secret_access_key = Seed._credentials(credentials_file)
 
         try:
-            service = Seed._backup_service()
+            service = Seed._backup_service(access_key_id, secret_access_key)
         except backup.BackupException as error:
             print(error)
             return -1
 
         if not service:
-            return False
+            return -1
+
+        indexes = list(range(100, 999))
+        for alias in service.aliases:
+            if alias.startswith('cryptnox'):
+                try:
+                    indexes.remove(int(alias))
+                except ValueError:
+                    continue
 
         while True:
-            name = ui.input_with_exit("Name of the seed on the KMS in HSM: ")
-            if re.match("^[a-zA-Z0-9:/_-]+$", name):
+            name = f'cryptnox{random.choice(indexes)}'
+            name = ui.input_with_exit(f'Name of the seed on the KMS in HSM ({name}): ',
+                                      required=False) or name
+            if re.match("^[a-zA-Z\\d:/_-]+$", name):
                 break
             print("Name of seed can only contain small and large letters, numbers and characters :/_-")
 
-        seed = card.generate_random_number(32)
+        seed = seed or card.generate_random_number(32)
         print("Backing up seed...")
         try:
             service.backup(name, seed)
@@ -114,15 +130,14 @@ class Seed(Command):
         backup_file.write_text(tabulate(data))
         print(f"\nBackup data also saved to: {backup_file}")
 
-        ui.print_warning("It is advised to delete your AWS access keys if you don't plan to use "
-                         "them.")
+        Seed._remove_credentials(credentials_file, service)
 
         return 0
 
     @staticmethod
-    def _backup_service() -> Union[None, backup.AWS]:
-        access_key_id = ui.input_with_exit("Access Key ID: ")
-        secret_access_key = ui.secret_with_exit("Secret Access Key: ")
+    def _backup_service(access_key_id: str = '', secret_access_key: str = '') -> Union[None, backup.AWS]:
+        access_key_id = access_key_id or ui.input_with_exit("Access Key ID: ")
+        secret_access_key = secret_access_key or ui.secret_with_exit("Secret Access Key: ")
 
         try:
             service = backup.AWS(access_key_id, secret_access_key)
@@ -134,7 +149,7 @@ class Seed(Command):
         for client, options in service.regions.items():
             for option in regions.values():
                 options.remove(option)
-            regions[client] = ui.option_input(options)
+            regions[client] = ui.option_input(options, default=DEFAULT_REGIONS[client])
 
         try:
             service.region = regions
@@ -153,6 +168,23 @@ class Seed(Command):
 
         return 0
 
+    @staticmethod
+    def _credentials(credential_file: Path = None) -> Tuple[str, str]:
+        access_key_id = ''
+        secret_access_key = ''
+        credential_file = credential_file or Path(get_download_folder()).joinpath('rootkey.csv')
+        try:
+            with open(credential_file, 'r') as file:
+                id_line = file.readline().strip()
+                secret_line = file.readline().strip()
+                if id_line.startswith('AWSAccessKeyId=') and secret_line.startswith('AWSSecretKey='):
+                    access_key_id = id_line[len('AWSAccessKeyId='):]
+                    secret_access_key = secret_line[len('AWSSecretKey='):]
+        except FileNotFoundError:
+            print('Credentials file not found.')
+
+        return access_key_id, secret_access_key
+
     def _dual_seed(self, card: cryptnoxpy.Card) -> int:
         try:
             card.dual_seed_public_key()
@@ -249,6 +281,8 @@ class Seed(Command):
 
     @staticmethod
     def _load_mnemonic(card: cryptnoxpy.Card, mnemonic: str, pin_code: str) -> None:
+        if len(mnemonic.split(' ')) not in (12, 24):
+            raise ValueError('Only mnemonic passphrases of length 12 and 24 are supported')
         seed = cryptos.bip39_mnemonic_to_seed(mnemonic)
         card.load_seed(seed, pin_code)
 
@@ -259,7 +293,17 @@ class Seed(Command):
         print("\nEnter the mnemonic root to recover (12 or 24 words):")
         mnemonic = ui.input_with_exit("> ")
         try:
-            Seed._load_mnemonic(card, mnemonic, pin_code)
+            seed = cryptos.bip39_mnemonic_to_seed(mnemonic)
+        except Exception as error:
+            print(error)
+            return -1
+
+        do_backup = ui.confirm('Do you wish to backup your seed to AWS KMS?')
+        try:
+            if do_backup:
+                Seed._backup(card, pin_code, seed)
+            else:
+                Seed._load_mnemonic(card, mnemonic, pin_code)
         except Exception as error:
             print(error)
             return -1
@@ -268,12 +312,26 @@ class Seed(Command):
 
         return 0
 
+    @staticmethod
+    def _remove_credentials(credentials_file: Path, service: backup.AWS) -> None:
+        try:
+            credentials_file.unlink()
+        except FileNotFoundError:
+            ui.print_warning("It is advised to delete your AWS access keys if you don't plan to use them.")
+        else:
+            service.delete_access_key()
+            print('Your credential file and credentials have been deleted.')
+
     @staticmethod
     def _restore(card: cryptnoxpy.Card) -> int:
         pin_code = Seed._get_pin_code(card)
-        service = Seed._backup_service()
+        credentials_file = Path(get_download_folder()).joinpath('rootkey.csv')
+        access_key_id, secret_access_key = Seed._credentials(credentials_file)
+
+        service = Seed._backup_service(access_key_id, secret_access_key)
         if not service:
             return -1
+
         name = ui.input_with_exit("Name of the seed on the KMS in HSM: ")
 
         print("Retrieving seed from service...")
@@ -296,8 +354,7 @@ class Seed(Command):
         Seed._load_mnemonic(card, mnemonic, pin_code)
         print("\nMnemonic loaded from backup service.")
 
-        ui.print_warning("It is advised to delete your AWS access keys if you don't plan to use "
-                         "them.")
+        Seed._remove_credentials(credentials_file, service)
 
         return 0
 
@@ -306,7 +363,17 @@ class Seed(Command):
         pin_code = Seed._get_pin_code(card)
         seed = card.generate_random_number(32)
         mnemonic = cryptos.entropy_to_words(seed)
-        Seed._load_mnemonic(card, mnemonic, pin_code)
+        do_backup = ui.confirm('Do you wish to backup your seed to AWS KMS?')
+
+        try:
+            if do_backup:
+                Seed._backup(card, pin_code, seed)
+            else:
+                Seed._load_mnemonic(card, mnemonic, pin_code)
+        except Exception as error:
+            print(error)
+            return -1
+
         print("\nMnemonic root :")
         print(mnemonic)
 
