@@ -1,18 +1,18 @@
 #!/usr/bin/python
-from .py2specials import *
-from .py3specials import *
+import base64
 import binascii
 import hashlib
-import re
-import base64
-import time
-import random
 import hmac
+import random
+import re
+import time
+
 from .ripemd import *
+from .specials import *
 
 # Elliptic curve parameters (secp256k1)
 
-P = 2**256 - 2**32 - 977
+P = 2 ** 256 - 2 ** 32 - 977
 N = 115792089237316195423570985008687907852837564279074904382605163141518161494337
 A = 0
 B = 7
@@ -30,6 +30,7 @@ def change_curve(p, n, a, b, gx, gy):
 def getG():
     return G
 
+
 # Extended Euclidean Algorithm
 
 
@@ -39,13 +40,12 @@ def inv(a, n):
     lm, hm = 1, 0
     low, high = a % n, n
     while low > 1:
-        r = high//low
-        nm, new = hm-lm*r, high-low*r
+        r = high // low
+        nm, new = hm - lm * r, high - low * r
         lm, low, hm, high = nm, new, lm, low
     return lm % n
 
 
-
 # JSON access (for pybtctool convenience)
 
 
@@ -65,13 +65,14 @@ def multiaccess(obj, prop):
     return [access(o, prop) for o in obj]
 
 
-def slice(obj, start=0, end=2**200):
+def slice(obj, start=0, end=2 ** 200):
     return obj[int(start):int(end)]
 
 
 def count(obj):
     return len(obj)
 
+
 _sum = sum
 
 
@@ -94,7 +95,7 @@ def jacobian_double(p):
     ysq = (p[1] ** 2) % P
     S = (4 * p[0] * ysq) % P
     M = (3 * p[0] ** 2 + A * p[2] ** 4) % P
-    nx = (M**2 - 2 * S) % P
+    nx = (M ** 2 - 2 * S) % P
     ny = (M * (S - nx) - 8 * ysq ** 2) % P
     nz = (2 * p[1] * p[2]) % P
     return (nx, ny, nz)
@@ -126,7 +127,7 @@ def jacobian_add(p, q):
 
 def from_jacobian(p):
     z = inv(p[2], P)
-    return ((p[0] * z**2) % P, (p[1] * z**3) % P)
+    return ((p[0] * z ** 2) % P, (p[1] * z ** 3) % P)
 
 
 def jacobian_multiply(a, n):
@@ -137,9 +138,9 @@ def jacobian_multiply(a, n):
     if n < 0 or n >= N:
         return jacobian_multiply(a, n % N)
     if (n % 2) == 0:
-        return jacobian_double(jacobian_multiply(a, n//2))
+        return jacobian_double(jacobian_multiply(a, n // 2))
     if (n % 2) == 1:
-        return jacobian_add(jacobian_double(jacobian_multiply(a, n//2)), a)
+        return jacobian_add(jacobian_double(jacobian_multiply(a, n // 2)), a)
 
 
 def fast_multiply(a, n):
@@ -149,6 +150,7 @@ def fast_multiply(a, n):
 def fast_add(a, b):
     return from_jacobian(jacobian_add(to_jacobian(a), to_jacobian(b)))
 
+
 # Functions for handling pubkey and privkey formats
 
 
@@ -162,91 +164,135 @@ def get_pubkey_format(pub):
         three = 3
         four = 4
 
-    if isinstance(pub, (tuple, list)): return 'decimal'
-    elif len(pub) == 65 and pub[0] == four: return 'bin'
-    elif len(pub) == 130 and pub[0:2] == '04': return 'hex'
-    elif len(pub) == 33 and pub[0] in [two, three]: return 'bin_compressed'
-    elif len(pub) == 66 and pub[0:2] in ['02', '03']: return 'hex_compressed'
-    elif len(pub) == 64: return 'bin_electrum'
-    elif len(pub) == 128: return 'hex_electrum'
-    else: raise Exception("Pubkey not in recognized format")
+    if isinstance(pub, (tuple, list)):
+        return 'decimal'
+    elif len(pub) == 65 and pub[0] == four:
+        return 'bin'
+    elif len(pub) == 130 and pub[0:2] == '04':
+        return 'hex'
+    elif len(pub) == 33 and pub[0] in [two, three]:
+        return 'bin_compressed'
+    elif len(pub) == 66 and pub[0:2] in ['02', '03']:
+        return 'hex_compressed'
+    elif len(pub) == 64:
+        return 'bin_electrum'
+    elif len(pub) == 128:
+        return 'hex_electrum'
+    else:
+        raise Exception("Pubkey not in recognized format")
 
 
 def encode_pubkey(pub, formt):
     if not isinstance(pub, (tuple, list)):
         pub = decode_pubkey(pub)
-    if formt == 'decimal': return pub
-    elif formt == 'bin': return b'\x04' + encode(pub[0], 256, 32) + encode(pub[1], 256, 32)
+    if formt == 'decimal':
+        return pub
+    elif formt == 'bin':
+        return b'\x04' + encode(pub[0], 256, 32) + encode(pub[1], 256, 32)
     elif formt == 'bin_compressed':
-        return from_int_to_byte(2+(pub[1] % 2)) + encode(pub[0], 256, 32)
-    elif formt == 'hex': return '04' + encode(pub[0], 16, 64) + encode(pub[1], 16, 64)
+        return from_int_to_byte(2 + (pub[1] % 2)) + encode(pub[0], 256, 32)
+    elif formt == 'hex':
+        return '04' + encode(pub[0], 16, 64) + encode(pub[1], 16, 64)
     elif formt == 'hex_compressed':
-        return '0'+str(2+(pub[1] % 2)) + encode(pub[0], 16, 64)
-    elif formt == 'bin_electrum': return encode(pub[0], 256, 32) + encode(pub[1], 256, 32)
-    elif formt == 'hex_electrum': return encode(pub[0], 16, 64) + encode(pub[1], 16, 64)
-    else: raise Exception("Invalid format!")
+        return '0' + str(2 + (pub[1] % 2)) + encode(pub[0], 16, 64)
+    elif formt == 'bin_electrum':
+        return encode(pub[0], 256, 32) + encode(pub[1], 256, 32)
+    elif formt == 'hex_electrum':
+        return encode(pub[0], 16, 64) + encode(pub[1], 16, 64)
+    else:
+        raise Exception("Invalid format!")
 
 
 def decode_pubkey(pub, formt=None):
     if not formt: formt = get_pubkey_format(pub)
-    if formt == 'decimal': return pub
-    elif formt == 'bin': return (decode(pub[1:33], 256), decode(pub[33:65], 256))
+    if formt == 'decimal':
+        return pub
+    elif formt == 'bin':
+        return (decode(pub[1:33], 256), decode(pub[33:65], 256))
     elif formt == 'bin_compressed':
         x = decode(pub[1:33], 256)
-        beta = pow(int(x*x*x+A*x+B), int((P+1)//4), int(P))
-        y = (P-beta) if ((beta + from_byte_to_int(pub[0])) % 2) else beta
+        beta = pow(int(x * x * x + A * x + B), int((P + 1) // 4), int(P))
+        y = (P - beta) if ((beta + from_byte_to_int(pub[0])) % 2) else beta
         return (x, y)
-    elif formt == 'hex': return (decode(pub[2:66], 16), decode(pub[66:130], 16))
+    elif formt == 'hex':
+        return (decode(pub[2:66], 16), decode(pub[66:130], 16))
     elif formt == 'hex_compressed':
         return decode_pubkey(safe_from_hex(pub), 'bin_compressed')
     elif formt == 'bin_electrum':
         return (decode(pub[:32], 256), decode(pub[32:64], 256))
     elif formt == 'hex_electrum':
         return (decode(pub[:64], 16), decode(pub[64:128], 16))
-    else: raise Exception("Invalid format!")
+    else:
+        raise Exception("Invalid format!")
+
 
 def get_privkey_format(priv):
-    if isinstance(priv, int_types): return 'decimal'
-    elif len(priv) == 32: return 'bin'
-    elif len(priv) == 33: return 'bin_compressed'
-    elif len(priv) == 64: return 'hex'
-    elif len(priv) == 66: return 'hex_compressed'
+    if isinstance(priv, int_types):
+        return 'decimal'
+    elif len(priv) == 32:
+        return 'bin'
+    elif len(priv) == 33:
+        return 'bin_compressed'
+    elif len(priv) == 64:
+        return 'hex'
+    elif len(priv) == 66:
+        return 'hex_compressed'
     else:
         bin_p = b58check_to_bin(priv)
-        if len(bin_p) == 32: return 'wif'
-        elif len(bin_p) == 33: return 'wif_compressed'
-        else: raise Exception("WIF does not represent privkey")
+        if len(bin_p) == 32:
+            return 'wif'
+        elif len(bin_p) == 33:
+            return 'wif_compressed'
+        else:
+            raise Exception("WIF does not represent privkey")
+
 
 def encode_privkey(priv, formt, vbyte=128):
     if not isinstance(priv, int_types):
         return encode_privkey(decode_privkey(priv), formt, vbyte)
-    if formt == 'decimal': return priv
-    elif formt == 'bin': return encode(priv, 256, 32)
-    elif formt == 'bin_compressed': return encode(priv, 256, 32)+b'\x01'
-    elif formt == 'hex': return encode(priv, 16, 64)
-    elif formt == 'hex_compressed': return encode(priv, 16, 64)+'01'
+    if formt == 'decimal':
+        return priv
+    elif formt == 'bin':
+        return encode(priv, 256, 32)
+    elif formt == 'bin_compressed':
+        return encode(priv, 256, 32) + b'\x01'
+    elif formt == 'hex':
+        return encode(priv, 16, 64)
+    elif formt == 'hex_compressed':
+        return encode(priv, 16, 64) + '01'
     elif formt == 'wif':
         return bin_to_b58check(encode(priv, 256, 32), int(vbyte))
     elif formt == 'wif_compressed':
         return bin_to_b58check(encode(priv, 256, 32) + b'\x01', int(vbyte))
-    else: raise Exception("Invalid format!")
+    else:
+        raise Exception("Invalid format!")
 
-def decode_privkey(priv,formt=None):
+
+def decode_privkey(priv, formt=None):
     if not formt: formt = get_privkey_format(priv)
-    if formt == 'decimal': return priv
-    elif formt == 'bin': return decode(priv, 256)
-    elif formt == 'bin_compressed': return decode(priv[:32], 256)
-    elif formt == 'hex': return decode(priv, 16)
-    elif formt == 'hex_compressed': return decode(priv[:64], 16)
-    elif formt == 'wif': return decode(b58check_to_bin(priv),256)
+    if formt == 'decimal':
+        return priv
+    elif formt == 'bin':
+        return decode(priv, 256)
+    elif formt == 'bin_compressed':
+        return decode(priv[:32], 256)
+    elif formt == 'hex':
+        return decode(priv, 16)
+    elif formt == 'hex_compressed':
+        return decode(priv[:64], 16)
+    elif formt == 'wif':
+        return decode(b58check_to_bin(priv), 256)
     elif formt == 'wif_compressed':
-        return decode(b58check_to_bin(priv)[:32],256)
-    else: raise Exception("WIF does not represent privkey")
+        return decode(b58check_to_bin(priv)[:32], 256)
+    else:
+        raise Exception("WIF does not represent privkey")
+
 
 def add_pubkeys(p1, p2):
     f1, f2 = get_pubkey_format(p1), get_pubkey_format(p2)
     return encode_pubkey(fast_add(decode_pubkey(p1, f1), decode_pubkey(p2, f2)), f1)
 
+
 def add_privkeys(p1, p2):
     f1, f2 = get_privkey_format(p1), get_privkey_format(p2)
     return encode_privkey((decode_privkey(p1, f1) + decode_privkey(p2, f2)) % N, f1)
@@ -256,11 +302,12 @@ def mul_privkeys(p1, p2):
     f1, f2 = get_privkey_format(p1), get_privkey_format(p2)
     return encode_privkey((decode_privkey(p1, f1) * decode_privkey(p2, f2)) % N, f1)
 
+
 def multiply(pubkey, privkey):
     f1, f2 = get_pubkey_format(pubkey), get_privkey_format(privkey)
     pubkey, privkey = decode_pubkey(pubkey, f1), decode_privkey(privkey, f2)
     # http://safecurves.cr.yp.to/twist.html
-    if not isinf(pubkey) and (pubkey[0]**3+B-pubkey[1]*pubkey[1]) % P != 0:
+    if not isinf(pubkey) and (pubkey[0] ** 3 + B - pubkey[1] * pubkey[1]) % P != 0:
         raise Exception("Point not on curve")
     return encode_pubkey(fast_multiply(pubkey, privkey), f1)
 
@@ -272,16 +319,20 @@ def divide(pubkey, privkey):
 
 def compress(pubkey):
     f = get_pubkey_format(pubkey)
-    if 'compressed' in f: return pubkey
-    elif f == 'bin': return encode_pubkey(decode_pubkey(pubkey, f), 'bin_compressed')
+    if 'compressed' in f:
+        return pubkey
+    elif f == 'bin':
+        return encode_pubkey(decode_pubkey(pubkey, f), 'bin_compressed')
     elif f == 'hex' or f == 'decimal':
         return encode_pubkey(decode_pubkey(pubkey, f), 'hex_compressed')
 
 
 def decompress(pubkey):
     f = get_pubkey_format(pubkey)
-    if 'compressed' not in f: return pubkey
-    elif f == 'bin_compressed': return encode_pubkey(decode_pubkey(pubkey, f), 'bin')
+    if 'compressed' not in f:
+        return pubkey
+    elif f == 'bin_compressed':
+        return encode_pubkey(decode_pubkey(pubkey, f), 'bin')
     elif f == 'hex_compressed' or f == 'decimal':
         return encode_pubkey(decode_pubkey(pubkey, f), 'hex')
 
@@ -296,18 +347,21 @@ def privkey_to_pubkey(privkey):
     else:
         return encode_pubkey(fast_multiply(G, privkey), f.replace('wif', 'hex'))
 
+
 privtopub = privkey_to_pubkey
 
 
 def privkey_to_address(priv, magicbyte=0):
     return pubkey_to_address(privkey_to_pubkey(priv), magicbyte)
+
+
 privtoaddr = privkey_to_address
 
 
 def neg_pubkey(pubkey):
     f = get_pubkey_format(pubkey)
     pubkey = decode_pubkey(pubkey, f)
-    return encode_pubkey((pubkey[0], (P-pubkey[1]) % P), f)
+    return encode_pubkey((pubkey[0], (P - pubkey[1]) % P), f)
 
 
 def neg_privkey(privkey):
@@ -315,6 +369,7 @@ def neg_privkey(privkey):
     privkey = decode_privkey(privkey, f)
     return encode_privkey((N - privkey) % N, f)
 
+
 def subtract_pubkeys(p1, p2):
     f1, f2 = get_pubkey_format(p1), get_pubkey_format(p2)
     k2 = decode_pubkey(p2, f2)
@@ -327,7 +382,6 @@ def subtract_privkeys(p1, p2):
     return encode_privkey((decode_privkey(p1, f1) - k2) % N, f1)
 
 
-
 # Hashes
 
 
@@ -340,16 +394,20 @@ def bin_hash160(string):
         digest = RIPEMD160(intermed).digest()
     return digest
 
+
 def hash160(string):
     return safe_hexlify(bin_hash160(string))
 
+
 def hex_to_hash160(s_hex):
     return hash160(binascii.unhexlify(s_hex))
 
+
 def bin_sha256(string):
     binary_data = string if isinstance(string, bytes) else bytes(string, 'utf-8')
     return hashlib.sha256(binary_data).digest()
 
+
 def sha256(string):
     return bytes_to_hex_string(bin_sha256(string))
 
@@ -395,10 +453,14 @@ def hash_to_int(x):
 
 def num_to_var_int(x):
     x = int(x)
-    if x < 253: return from_int_to_byte(x)
-    elif x < 65536: return from_int_to_byte(253)+encode(x, 256, 2)[::-1]
-    elif x < 4294967296: return from_int_to_byte(254) + encode(x, 256, 4)[::-1]
-    else: return from_int_to_byte(255) + encode(x, 256, 8)[::-1]
+    if x < 253:
+        return from_int_to_byte(x)
+    elif x < 65536:
+        return from_int_to_byte(253) + encode(x, 256, 2)[::-1]
+    elif x < 4294967296:
+        return from_int_to_byte(254) + encode(x, 256, 4)[::-1]
+    else:
+        return from_int_to_byte(255) + encode(x, 256, 8)[::-1]
 
 
 # WTF, Electrum?
@@ -410,18 +472,19 @@ def electrum_sig_hash(message):
 def random_key():
     # Gotta be secure after that java.SecureRandom fiasco...
     entropy = random_string(32) \
-        + str(random.randrange(2**256)) \
-        + str(int(time.time() * 1000000))
+              + str(random.randrange(2 ** 256)) \
+              + str(int(time.time() * 1000000))
     return sha256(entropy)
 
 
 def random_electrum_seed():
-    #entropy = os.urandom(32) \ # fails in Python 3, hence copied from random_key()
+    # entropy = os.urandom(32) \ # fails in Python 3, hence copied from random_key()
     entropy = random_string(32) \
-        + str(random.randrange(2**256)) \
-        + str(int(time.time() * 1000000))
+              + str(random.randrange(2 ** 256)) \
+              + str(int(time.time() * 1000000))
     return sha256(entropy)[:32]
 
+
 # Encodings
 
 def b58check_to_bin(inp):
@@ -445,6 +508,7 @@ def hex_to_b58check(inp, magicbyte=0):
 def b58check_to_hex(inp):
     return safe_hexlify(b58check_to_bin(inp))
 
+
 def pubkey_to_hash(pubkey):
     if isinstance(pubkey, (list, tuple)):
         pubkey = encode_pubkey(pubkey, 'bin')
@@ -452,13 +516,16 @@ def pubkey_to_hash(pubkey):
         return bin_hash160(binascii.unhexlify(pubkey))
     return bin_hash160(pubkey)
 
+
 def pubkey_to_hash_hex(pubkey):
     return safe_hexlify(pubkey_to_hash(pubkey))
 
+
 def pubkey_to_address(pubkey, magicbyte=0):
     pubkey_hash = pubkey_to_hash(pubkey)
     return bin_to_b58check(pubkey_hash, magicbyte)
 
+
 pubtoaddr = pubkey_to_address
 
 
@@ -469,6 +536,7 @@ def is_privkey(priv):
     except:
         return False
 
+
 def is_pubkey(pubkey):
     try:
         get_pubkey_format(pubkey)
@@ -482,8 +550,8 @@ def is_pubkey(pubkey):
 
 def encode_sig(v, r, s):
     vb, rb, sb = from_int_to_byte(v), encode(r, 256), encode(s, 256)
-    
-    result = base64.b64encode(vb+b'\x00'*(32-len(rb))+rb+b'\x00'*(32-len(sb))+sb)
+
+    result = base64.b64encode(vb + b'\x00' * (32 - len(rb)) + rb + b'\x00' * (32 - len(sb)) + sb)
     return result if is_python2 else str(result, 'utf-8')
 
 
@@ -491,6 +559,7 @@ def decode_sig(sig):
     bytez = base64.b64decode(sig)
     return from_byte_to_int(bytez[0]), decode(bytez[1:33], 256), decode(bytez[33:], 256)
 
+
 # https://tools.ietf.org/html/rfc6979#section-3.2
 
 
@@ -499,22 +568,21 @@ def deterministic_generate_k(msghash, priv):
     k = b'\x00' * 32
     priv = encode_privkey(priv, 'bin')
     msghash = encode(hash_to_int(msghash), 256, 32)
-    k = hmac.new(k, v+b'\x00'+priv+msghash, hashlib.sha256).digest()
+    k = hmac.new(k, v + b'\x00' + priv + msghash, hashlib.sha256).digest()
     v = hmac.new(k, v, hashlib.sha256).digest()
-    k = hmac.new(k, v+b'\x01'+priv+msghash, hashlib.sha256).digest()
+    k = hmac.new(k, v + b'\x01' + priv + msghash, hashlib.sha256).digest()
     v = hmac.new(k, v, hashlib.sha256).digest()
     return decode(hmac.new(k, v, hashlib.sha256).digest(), 256)
 
 
 def ecdsa_raw_sign(msghash, priv):
-
     z = hash_to_int(msghash)
     k = deterministic_generate_k(msghash, priv)
 
     r, y = fast_multiply(G, k)
-    s = inv(k, N) * (z + r*decode_privkey(priv)) % N
+    s = inv(k, N) * (z + r * decode_privkey(priv)) % N
 
-    v, r, s = 27+((y % 2) ^ (0 if s * 2 < N else 1)), r, s if s * 2 < N else N - s
+    v, r, s = 27 + ((y % 2) ^ (0 if s * 2 < N else 1)), r, s if s * 2 < N else N - s
     if 'compressed' in get_privkey_format(priv):
         v += 4
     return v, r, s
@@ -523,8 +591,8 @@ def ecdsa_raw_sign(msghash, priv):
 def ecdsa_sign(msg, priv, coin):
     v, r, s = ecdsa_raw_sign(electrum_sig_hash(msg), priv)
     sig = encode_sig(v, r, s)
-    assert ecdsa_verify(msg, sig, 
-        privtopub(priv), coin), "Bad Sig!\t %s\nv = %d\n,r = %d\ns = %d" % (sig, v, r, s)
+    assert ecdsa_verify(msg, sig,
+                        privtopub(priv), coin), "Bad Sig!\t %s\nv = %d\n,r = %d\ns = %d" % (sig, v, r, s)
     return sig
 
 
@@ -534,7 +602,7 @@ def ecdsa_raw_verify(msghash, vrs, pub):
     w = inv(s, N)
     z = hash_to_int(msghash)
 
-    u1, u2 = z*w % N, r*w % N
+    u1, u2 = z * w % N, r * w % N
     x, y = fast_add(fast_multiply(G, u1), fast_multiply(decode_pubkey(pub), u2))
     return bool(r == x and (r % N) and (s % N))
 
@@ -556,12 +624,12 @@ def ecdsa_verify(msg, sig, pub, coin):
 def ecdsa_raw_recover(msghash, vrs):
     v, r, s = vrs
     x = r
-    xcubedaxb = (x*x*x+A*x+B) % P
-    beta = pow(xcubedaxb, (P+1)//4, P)
+    xcubedaxb = (x * x * x + A * x + B) % P
+    beta = pow(xcubedaxb, (P + 1) // 4, P)
     y = beta if v % 2 ^ beta % 2 else (P - beta)
     # If xcubedaxb is not a quadratic residue, then r cannot be the x coord
     # for a point on the curve, and so the sig is invalid
-    if (xcubedaxb - y*y) % P != 0 or not (r % N) or not (s % N):
+    if (xcubedaxb - y * y) % P != 0 or not (r % N) or not (s % N):
         return False
     z = hash_to_int(msghash)
     Gz = jacobian_multiply((Gx, Gy, 1), (N - z) % N)
@@ -576,28 +644,30 @@ def ecdsa_raw_recover(msghash, vrs):
 
 
 def ecdsa_recover(msg, sig):
-    v,r,s = decode_sig(sig)
-    Q = ecdsa_raw_recover(electrum_sig_hash(msg), (v,r,s))
+    v, r, s = decode_sig(sig)
+    Q = ecdsa_raw_recover(electrum_sig_hash(msg), (v, r, s))
     return encode_pubkey(Q, 'hex_compressed') if v >= 31 else encode_pubkey(Q, 'hex')
 
 
-
-# add/subtract 
-def add(p1,p2):
+# add/subtract
+def add(p1, p2):
     if is_privkey(p1):
         return add_privkeys(p1, p2)
     else:
         return add_pubkeys(p1, p2)
 
-def subtract(p1,p2):
+
+def subtract(p1, p2):
     if is_privkey(p1):
         return subtract_privkeys(p1, p2)
     else:
         return subtract_pubkeys(p1, p2)
 
+
 hash160Low = b'\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00'
 hash160High = b'\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff'
 
+
 def magicbyte_to_prefix(magicbyte):
     first = bin_to_b58check(hash160Low, magicbyte=magicbyte)[0]
     last = bin_to_b58check(hash160High, magicbyte=magicbyte)[0]
